cccccc Subroutine for Kinematics   ccccccccccccccccccccccccccccccccc


      subroutine reaction_in_flight_2( M_i1, p_i1, v_i1, M_i2, p_i2, v_i2, 
     &		m1, m2, p1, p2, v1, v2, iflag )
      implicit none
      real p_i1, M_i1, v_i1(3), p_i2, M_i2, v_i2(3), M_inv, p_inv, v_inv(3)
	real m1, m2, p1, p2, v1(3), v2(3)
      integer iflag, j
c
	p_inv = 0.
	Do j = 1, 3
	   v_inv(j) = p_i1 * v_i1(j) + p_i2 * v_i2(j)
	   p_inv = p_inv + v_inv(j)**2
	EndDo
	M_inv = sqrt(
     &	( sqrt(M_i1**2 + p_i1**2) + sqrt(M_i2**2 + p_i2**2) )**2
     &	- p_inv  )
	p_inv = sqrt(p_inv)
	Do j = 1, 3
	   v_inv(j) = v_inv(j) / p_inv
	EndDo
      call decay_2_in_flight( M_inv, p_inv, v_inv, 
     &		m1, m2, p1, p2, v1, v2, iflag )
      if(iflag.eq.1) return
      call relat( p1, m1, v1, p_inv, M_inv, v_inv )
      call relat( p2, m2, v2, p_inv, M_inv, v_inv )
c
      return
      end
      


      subroutine decay_2_in_flight( M_inv, p_ini, v_ini, 
     &		m1, m2, p1, p2, v1, v2, iflag )
      implicit none
c   generate 2 body decay    M_inv( p ) --> m1( p1 ) + m2( p2 )
      real p_ini, M_inv, m1, m2, p1, p2, v_ini(3), v1(3), v2(3)
      integer iflag
c
      call decay_2_at_rest( M_inv, m1, m2, p1, p2, v1, v2, iflag )
      if(iflag.eq.1) return
      call relat( p1, m1, v1, p_ini, M_inv, v_ini )
      call relat( p2, m2, v2, p_ini, M_inv, v_ini )
c
      return
      end
      


      subroutine decay_2_at_rest( M_inv, m1, m2, p1, p2, v1, v2, iflag )
      implicit none
c   generate 2 body decay    M_inv( p ) --> m1( p1 ) + m2( p2 )
      real M_inv, m1, m2, p1, p2, v1(3), v2(3), Q
      integer j, iflag
      real eps / 1.0e-5/
c common for debug
	logical debug_F
      integer ev_NUM, ev_error
      common /DeBugging/ debug_F, ev_NUM, ev_error
c begin
	Q = M_inv - (m1 + m2)
      if(M_inv.le.m1+m2+EpS) then
         if(iflag .ne. -1) then
		write(*,*) ' warning +++++++ 2-body decay faild '
		write(*,*) 'M_inv,mass1,mass2,Q', M_inv,m1,m2,Q, ev_NUM
	   endif
         p1 = 0.
         p2 = 0.
         iflag = 1
         return
      else
         iflag = 0
      endif
      p1 = sqrt( (M_inv - m1 - m2) * (M_inv - m1 + m2)
     &         * (M_inv + m1 - m2) * (M_inv + m1 + m2) )
     &   / (2.*M_inv)
      p2 = p1
      call sphrrn( v1 )
      Do j = 1, 3
        v2(j) = -v1(j)
      EndDo
c
      return
      end



      subroutine decay_3_in_flight( M_inv, p, v, mass1, mass2, mass3, 
     &            p1, p2, p3, v1, v2, v3, iflag )
      implicit NONE
c input
      real M_inv, p, v(3), mass1, mass2, mass3
c output
      real p1, v1(3), p2, v2(3), p3, v3(3)
      integer iflag
c local 
      real E, b_cm
c begin
      E = sqrt(M_inv**2 + p**2 )
      b_cm = p / E
      call decay_3_at_rest( M_inv, mass1, mass2, mass3, 
     &            p1, p2, p3, v1, v2, v3, iflag )
      if(iflag.eq.1) return
      call relat2(p1,mass1,v1,b_cm,v)
      call relat2(p2,mass2,v2,b_cm,v)
      call relat2(p3,mass3,v3,b_cm,v)
      return
      end



      subroutine decay_3_at_rest(M_inv,mass1,mass2,mass3,p1,p2,p3,v1,v2,v3,iflag)
c> 3-body decay simulation
c> only its phase space is considered
c> input
c>  M_inv : mass of mother particle
c>  mass1 : mass of the doughter particle No.1
c>  mass2 : mass of the doughter particle No.2
c>  mass3 : mass of the doughter particle No.3
c> output
c>  p1 : momentum of the particle No.1
c>  p2,p3 : same as p1
c>  v1 : normal vector of the particle No.1
c>  v2,v3 : same as v1
      implicit none
c var
      real v1(3),v2(3),v3(3),M_inv,mass1,mass2,mass3,p1,p2,p3
      real c1_3, Q
      integer iflag
      real eps / 1.0e-5/
      logical ph3_OK
c begin
      Q = M_inv - mass1 - mass2 - mass3
      if(Q .lt. eps) then
         if(iflag .ne. -1) then
		write(*,*) ' warning +++++++ 3-body decay faild '
		write(*,*) 'M_inv,mass1,mass2,mass3,Q', 
     &		M_inv,mass1,mass2,mass3,Q
	   endif
         p1 = 0.
         p2 = 0.
         p3 = 0.
         iflag = 1
         return
      else
         iflag = 0
      endif
      ph3_OK = .false.
      Do while( .not. ph3_OK )
         call phase_3_trial(Q, mass1, mass2, mass3, p1, p2, p3, c1_3, ph3_OK)
      EndDo
      call v3set(c1_3,v1,v2,v3,p1,p2,p3)
      return
      end



      subroutine phase_3_trial(Q, mass1, mass2, mass3, p1, p2, p3, c1_3, ph3_OK)
      implicit none
c     3 body phase generation according to Dalitz method
c input 
      real Q, mass1, mass2, mass3, p1, p2, p3
c output
      real c1_3 ! opening angle (cosine) between v1 and v3
      logical ph3_OK
c local 
      real T1, T2, T3, rn
      real a/1.154700538E0/ ! = 2/sqrt(3)
      real b/1.732050808E0/ ! = sqrt(3)
c begin
c	write(*,*) 'pst', Q, mass1, mass2, mass3
10    T3 = Q * rn()
      T1 = Q * rn()
      if( T1 .gt. Q - T3 ) then
          T1 = T1 + T3 - Q
          T3 = Q - T3
      endif
      T2 = Q - T1 - T3
c	write(*,*) 'TTT', T1, T2, T3
      if( T1 .lt. 0. ) go to 10
      if( T2 .lt. 0. ) go to 10
      if( T3 .lt. 0. ) go to 10
      p1 = sqrt(T1*(T1+2.*mass1))
      p2 = sqrt(T2*(T2+2.*mass2))
      p3 = sqrt(T3*(T3+2.*mass3))
      if( abs(p1**2 + p3**2 - p2**2) .gt. abs(2.*p1*p3) ) then
         c1_3 = 100.
         ph3_OK = .false.
      else
c	   if(ev_NUM .ge. ev_error) write(*,*) 'p1*p3', p1*p3
         c1_3 = (p1**2 + p3**2 - p2**2) / (2.*p1*p3)
c	   if(ev_NUM .ge. ev_error) write(*,*) 'c1_3', c1_3, p1, p2, p3
         ph3_OK = .true.
      endif
      return
      end



      subroutine v3set(c1_3,v1,v2,v3,p1,p2,p3)
c set vectors v1, v2 and v3 uniformly to the space 
c where the opening angle (cosQ) between v1.v3 = c1_3 
c and vector sum p1.v1 + p2.v2 + p3.v3 = 0
c input 
c c:  opening angle (cosine) between vector v1 and v2
      implicit none
c var
      real v1(3),v2(3),v3(3),p1,p2,p3
      real c1_3 
c begin
      call sphrrn( v3 )
      call v3set_v3keep(c1_3,v1,v2,v3,p1,p2,p3)
      return
      end



      subroutine v3set_v3keep(c1_3,v1,v2,v3,p1,p2,p3)
c set vectors v1, v2 and v3 uniformly to the space 
c where the opening angle (cosQ) between v1.v3 = c 
c and vector sum p1.v1 + p2.v2 + p3.v3 = 0
c input 
c c:  opening angle (cosine) between vector v1 and v3
      implicit none
c var
      real v1(3),v2(3),v3(3),p1,p2,p3, v1s(3)
      real c1_3, s1_3, r1, r2
      integer j
      real pi314 /3.141592653589793e0/
c begin
	s1_3 = 1. - c1_3**2
	if(s1_3 .lt. 0.) then
	  c1_3 = 1.
	  s1_3 = 0.
	  if(c1_3 .gt. 0.) then
          Do j = 1, 3
            v1(j) = v3(j) 
          enddo
	  else
          Do j = 1, 3
            v1(j) = - v3(j) 
          enddo
	  endif
	  go to 10
	else
        s1_3 = sqrt( s1_3 )   
	endif
cc	write(*,*) 'v3, c1_3', v3, c1_3, (p1**2 + p3**2 - p2**2) / (2.*p1*p3)
	call get_arb_orth(v3, v1s)
cc	write(*,*) 'v1g', v1s
      Do j = 1, 3
        v1(j) = - ( c1_3 * v3(j) + s1_3 * v1s(j) )
      enddo
cc	write(*,*) 'sprod3 = c1_3', sprod( v1, v3 )
10    Do j = 1, 3
        v2(j) = - ( p1*v1(j) + p3*v3(j) ) / p2
      enddo
	r1 = sqrt( v1(1)**2 + v1(2)**2 + v1(3)**2 )
	r2 = sqrt( v2(1)**2 + v2(2)**2 + v2(3)**2 )
cc	write(*,*) 'r1, r2', r1, r2
      Do j = 1, 3
        v1(j) = v1(j) / r1
        v2(j) = v2(j) / r2
      enddo
      return
      end



	subroutine get_arb_orth(v, v1)
	implicit none
c var
c input
	real v(3)
c output
	real v1(3)
c local
	real v1s(3), v2s(3), r, sp, phi, c2, s2
      real pi314 /3.141592653589793e0/
	integer j
c external
	real rn, sprod
c begin
      if(abs(v(1)) .lt. 0.5) then
        v1s(1) = 1.
        v1s(2) = 0.
        v1s(3) = 0.
      else if(abs(v(2)) .lt. 0.5) then
        v1s(1) = 0.
        v1s(2) = 1.
        v1s(3) = 0.
      else 
        v1s(1) = 0.
        v1s(2) = 0.
        v1s(3) = 1.
      endif
      sp = sprod( v, v1s )
cc	write(*,*) 'sprod1', sprod( v, v1s )
      r = 0.
      Do j = 1, 3
        v1s(j) = v1s(j) - sp * v(j)
        r = r + v1s(j)**2
      enddo
      r = sqrt( r )
      Do j = 1, 3
        v1s(j) = v1s(j) / r
      enddo
      call vprod( v, v1s, v2s )
cc	write(*,*) 'v  ', v, sp
cc	write(*,*) 'v1s', v1s
cc	write(*,*) 'v2s', v2s
      phi = 2. * pi314 * rn()
      c2 = cos(phi)
      s2 = sin(phi) 
	r = 0.
      Do j = 1, 3
        v1(j) = c2*v1s(j) + s2*v2s(j)  
      enddo
	r = sqrt( v1(1)**2 + v1(2)**2 + v1(3)**2 )
cc	write(*,*) 'sprod2', sprod( v, v1s ), r
      Do j = 1, 3
        v1(j) = v1(j) / r
      enddo
      return
      end



      subroutine decay_4_in_flight( M_inv, p, v, mass1, mass2, mass3, mass4,
     &            p1, p2, p3, p4, v1, v2, v3, v4, iflag )
      implicit NONE
c input
      real M_inv, p, v(3), mass1, mass2, mass3, mass4
c output
      real p1, v1(3), p2, v2(3), p3, v3(3), p4, v4(3)
      integer iflag
c local 
      real E, b_cm
c begin
      E = sqrt(M_inv**2 + p**2 )
      b_cm = p / E
      call decay_4_at_rest( M_inv, mass1, mass2, mass3, mass4,
     &            p1, p2, p3, p4, v1, v2, v3, v4, iflag )
      if(iflag.eq.1) return
      call relat2(p1,mass1,v1,b_cm,v)
      call relat2(p2,mass2,v2,b_cm,v)
      call relat2(p3,mass3,v3,b_cm,v)
      call relat2(p4,mass4,v4,b_cm,v)
      return
      end



      subroutine decay_4_at_rest( M_inv, mass1, mass2, mass3, mass4,
     &            p1, p2, p3, p4, v1, v2, v3, v4, iflag )
      implicit NONE
c input
      real M_inv, mass1, mass2, mass3, mass4
c output
      real p1, v1(3), p2, v2(3), p3, v3(3), p4, v4(3)
      integer iflag
c common
      real*8 Q_tot, M_A, Q_A, M4, T4, T4_max, ro_max
      common /real8c/ Q_tot, M_A, Q_A, M4, T4, T4_max, ro_max
c local
      real rn, Q3, b_A, c1_3, weight
      logical ph4_OK, ph3_OK
c begin
c Comment:
c A simplified pahse space weight for particle No. 4 is applied.
c For the accurate calculation, the weight must be modified to: 
c weight =  2 * \pi**2  * sqrt( T4 * (T4 + 2.*M4) ) * 
c           \int{M3}{E3_max}dE3 sqrt{ E3**2 - M3**2 }
c                        * { (E2cm - M1 - M2) * (E2cm - M1 + M2) 
c                          * (E2cm + M1 - M2) * (E2cm + M1 + M2) }**{1/2} / E2cm
c where 
c E3cm = sqrt((M_inv-M4)**2 - 2*M_inv*T4 ), 
c E3_max = (E3cm**2 - M3**2 - (M1 + M2)**2) / (2*E3cm)
c E2cm = sqrt{(E3cm-M3)**2 - 2*E3cm*T3}, 
c Q_tot = M_inv - M1 - M2 - M3 - M4
c M_A = M1 + M2 + M3
c T4 = E4 - M4
c In here, we simply assumed that the 3 body phase space is propotional to 
c the Q-value in the center of mass for particle No. 1, 2 and 3,
c which means that the shape of the boundary of Dalitz plot is propotional 
c to the 3 body Q_value independent of the energy (this is approximately true).
c
      call search_4_phase_max(M_inv, mass1, mass2, mass3, mass4, iflag)
      if(iflag .eq. 1) then
         p1 = 0.
         p2 = 0.
         p3 = 0.
         p4 = 0.
         return
      endif
      ph4_OK = .false.
      Do while ( .not. ph4_OK )
         T4 = T4_max * rn()
         Q_A = sqrt((M_A + Q_tot)**2 - 2.d0*(M_A + Q_tot + M4)*T4 ) - M_A
         weight = sqrt( T4 * (T4 + 2.*M4) ) * Q_A**2 / ro_max
         if( rn() .lt. weight ) ph4_OK = .true.
      EndDo
c
      p4 = sqrt( T4 * (T4 + 2.*M4) )
      b_A = - p4 / (M_A + Q_tot - T4) 
      call sphrrn( v4 )
c
      Q3 = Q_A
      ph3_OK = .false.
      Do while( .not. ph3_OK )
         call phase_3_trial(Q3, mass1, mass2, mass3, p1, p2, p3, c1_3, ph3_OK)
      EndDo
      call v3set(c1_3,v1,v2,v3,p1,p2,p3)
c
      call relat2(p1,mass1,v1,b_A,v4)
      call relat2(p2,mass2,v2,b_A,v4)
      call relat2(p3,mass3,v3,b_A,v4)
c
      return
      end



      subroutine search_4_phase_max(M_inv, mass1, mass2, mass3, mass4, iflag)
      implicit NONE
c i/o
      real M_inv, mass1, mass2, mass3, mass4
      integer iflag
c common
      real*8 Q_tot, M_A, Q_A, M4, T4, T4_max, ro_max
      common /real8c/ Q_tot, M_A, Q_A, M4, T4, T4_max, ro_max
c local
      real d_ro, d_ro_low, d_ro_high, diff, T4_ro_max
      real T4_low, T4_high
      real eps / 1.0e-5/
c begin
      M_A = mass1 + mass2 + mass3
      M4  = mass4
      Q_tot   = M_inv - M_A - M4
      if(Q_tot .lt. eps) then
         if(iflag .ne. -1) then
	      write(*,*) ' warning +++++++ 4-body decay faild '
            write(*,*) 'M_inv, mass1, mass2, mass3, mass4',
     &            M_inv, mass1, mass2, mass3, mass4
	   endif
         iflag = 1
         return
      else
         iflag = 0
      endif
      T4_max = (Q_tot*(M_A + Q_tot/2.d0)) / (M_A + M4 + Q_tot) * (1.d0-1.d-9)
      T4_low = T4_max * 0.001
      T4 = T4_low
      Q_A = sqrt((M_A + Q_tot)**2 - 2.d0*(M_A + Q_tot + M4)*T4 ) - M_A
      d_ro = (T4 + M4) * Q_A 
     &        - 2.d0 * (M_A + Q_tot + M4) * ( T4 * (T4 + 2.*M4) ) / (Q_A + M_A)
      d_ro_low = d_ro

      T4_high = T4_max * 0.99
      T4 = T4_high
      Q_A = sqrt((M_A + Q_tot)**2 - 2.d0*(M_A + Q_tot + M4)*T4 ) - M_A
      d_ro = (T4 + M4) * Q_A 
     &        - 2.d0 * (M_A + Q_tot + M4) * ( T4 * (T4 + 2.*M4) ) / (Q_A + M_A)
      d_ro_high = d_ro
      diff = (T4_high - T4_low) / T4_max
c
      Do while (diff .gt. 0.005)
         T4 = (T4_low + T4_high) / 2.
         Q_A = sqrt((M_A + Q_tot)**2 - 2.d0*(M_A + Q_tot + M4)*T4 ) - M_A
         d_ro = (T4 + M4) * Q_A 
     &        - 2.d0 * (M_A + Q_tot + M4) * ( T4 * (T4 + 2.*M4) ) / (Q_A + M_A)
         if(d_ro .gt. 0.) then
            T4_low = T4
         else
            T4_high = T4
         endif 
         diff = (T4_high - T4_low) / T4_max
      enddo
      T4_ro_max = T4
      ro_max = sqrt( T4 * (T4 + 2.*M4) ) * Q_A**2
      return
      end



c 3 body reaction at rest



      subroutine reaction_at_rest_3(M_nucl, M_proj,
     &            mass1, mass2, mass3, p1, p2, p3, v1, v2, v3,
     &            N_body, N_pickup, iflag)
      implicit NONE
c> 3-body reaction in 3 body phase space with nuclear form factor 
c> input
c>  M_inv : total mass (^4He + kaon, ^4He + Sigma^- etc.)
c>  mass1 : mass of the doughter particle No.1
c>  mass2 : mass of the doughter particle No.2
c>  mass3 : mass of the doughter nuclei
c>  N_body : define nuclear form factor by the number of nucleon
c> output
c>  p1 : momentum of the particle No.1
c>  p2,p3 : same as p1
c>  v1 : normal vector of the particle No.1
c>  v2,v3 : same as v1
c var
      real v1(3),v2(3),v3(3),M_inv,mass1,mass2,mass3,p1,p2,p3
      real M_nucl, p_nucl, v_nucl(3), M_proj
      integer N_body, N_pickup, iflag
c begin
      p_nucl = 0.
c      write(*,*) '*** reaction_at_cm_3'
	M_inv = M_nucl + M_proj
      call reaction_at_cm_3(M_inv, M_nucl, p_nucl, v_nucl,
     &            mass1, mass2, mass3, p1, p2, p3, v1, v2, v3,
     &            N_body, N_pickup, iflag)
*      write(*,*) '--- reaction_at_cm_3'
      return
      end




      subroutine reaction_at_rest_4(M_nucl, M_proj,
     &            mass1, mass2, mass3, mass4,
     &            p1, p2, p3, p4, v1, v2, v3, v4,
     &            N_body, N_pickup, iflag)
      implicit NONE
c> 3-body reaction in 3 body phase space with nuclear form factor 
c> input
c>  M_inv : total mass (^4He + kaon, ^4He + Sigma^- etc.)
c>  mass1 : mass of the doughter particle No.1
c>  mass2 : mass of the doughter particle No.2
c>  mass3 : mass of the doughter particle No.3
c>  mass4 : mass of the doughter nuclei
c>  N_body : define nuclear form factor by the number of nucleon
c> output
c>  p1 : momentum of the particle No.1
c>  p2,p3 : same as p1
c>  v1 : normal vector of the particle No.1
c>  v2,v3 : same as v1
c var
      real v1(3),v2(3),v3(3),v4(3),M_inv
      real mass1,mass2,mass3,mass4,p1,p2,p3,p4
      real M_nucl, p_nucl, v_nucl(3), M_proj
      integer N_body, N_pickup, iflag
c begin
      p_nucl = 0.
	M_inv = M_nucl + M_proj
      call reaction_at_cm_4(M_inv, M_nucl, p_nucl, v_nucl,
     &            mass1, mass2, mass3, mass4, 
     &            p1, p2, p3, p4, v1, v2, v3, v4,
     &            N_body, N_pickup, iflag)
      return
      end



      subroutine reaction_in_flight_3(
     &		M_nucl, p_nucl, v_nucl, M_proj, p_proj, v_proj,
     &            mass1, mass2, mass3, p1, p2, p3,
     &            v1, v2, v3, N_body, N_pickup, iflag)
      implicit NONE
      real v1(3),v2(3),v3(3),mass1,mass2,mass3,p1,p2,p3
      real M_inv
      real  p_nucl, M_nucl, v_nucl(3), p_proj, M_proj, v_proj(3)
      real  beta_cm, v_cm(3), beta_N, E_nucl
      integer N_body, N_pickup, iflag, j
c common for debug
	logical debug_F
      integer ev_NUM, ev_error
      common /DeBugging/ debug_F, ev_NUM, ev_error
c begin
	if( p_nucl .eq. 0. and. p_proj .eq. 0. ) then
c>	   if(debug_F .and. (ev_NUM .ge. ev_error)) then
c>		write(*,*) 'at rest reaction : M_inv', M_inv
c>	   endif
         call reaction_at_rest_3( M_nucl, M_proj, 
     &            mass1, mass2, mass3, p1, p2, p3,
     &            v1, v2, v3, N_body, N_pickup, iflag)
	   return
	endif
c ... move to nuclear center system
      call transform2cm(p_nucl,M_nucl,v_nucl,p_proj,M_proj,v_proj,beta_cm,v_cm)
      E_nucl = sqrt(M_nucl**2 + p_nucl**2)
      beta_N = p_nucl / E_nucl
      M_inv = E_nucl + sqrt(M_proj**2 +  p_proj**2)
c      if(ev_NUM .ge. ev_error) write(*,*) 'M_inv', M_inv, M_nucl, p_nucl, v_nucl
      call reaction_at_cm_3(M_inv, M_nucl, p_nucl, v_nucl,
     &            mass1, mass2, mass3, p1, p2, p3, v1, v2, v3,
     &            N_body, N_pickup, iflag)
c	   if(debug_F .and. (ev_NUM .ge. ev_error)) then
c		write(*,*) 'here 1',  v3, iflag
c	   endif
c if reaction does not occur in this energy it forces the original 
c particles to be scattered in s-wave
      if(iflag .eq. 1) then
c>	   if(debug_F .and. (ev_NUM .ge. ev_error)) then
c>		write(*,*) 'iflag=1', ev_NUM, ' : ', v_nucl
c>	   endif
         call sphrrn( v_nucl )
         Do j = 1, 3
            v_proj(j) = -v_nucl(j)
         EndDo
         call relat2(p_nucl, M_nucl, v_nucl, beta_cm, v_cm)
         call relat2(p_proj, M_proj, v_proj, beta_cm, v_cm)
         return
      endif
      call relat2(p1, mass1, v1, beta_cm, v_cm)
      call relat2(p2, mass2, v2, beta_cm, v_cm)
      call relat2(p3, mass3, v3, beta_cm, v_cm)
c	   if(debug_F .and. (ev_NUM .ge. ev_error)) then
c		write(*,*) 'here 2 p1',  p1, v1
c		write(*,*) 'here 2 p2',  p2, v2
c		write(*,*) 'here 2 p3',  p3, v3
c	   endif
      return
      end



      subroutine reaction_in_flight_4(
     &		M_nucl,p_nucl,v_nucl, M_proj,p_proj,v_proj,
     &            mass1, mass2, mass3, mass4, p1, p2, p3, p4,
     &            v1, v2, v3, v4, N_Body, N_pickup, iflag)
      implicit NONE
      real v1(3),v2(3),v3(3),v4(3),mass1,mass2,mass3,mass4
      real p1,p2,p3,p4
      real M_inv
      real  p_nucl, M_nucl, v_nucl(3), p_proj, M_proj, v_proj(3)
      real  beta_cm, v_cm(3), beta_N, E_nucl
      integer N_body, N_pickup, iflag, j
c begin
	if( p_nucl .eq. 0. and. p_proj .eq. 0. ) then
         call reaction_at_rest_4(M_nucl, M_proj, 
     &            mass1, mass2, mass3, mass4,
     &            p1, p2, p3, p4, v1, v2, v3, v4,
     &            N_body, N_pickup, iflag)
	   return
	endif
c ... move to nuclear center system
      call transform2cm(p_nucl,M_nucl,v_nucl,p_proj,M_proj,v_proj,
     &		beta_cm,v_cm)
      E_nucl = sqrt(M_nucl**2 + p_nucl**2)
      beta_N = p_nucl / E_nucl
      M_inv = E_nucl + sqrt(M_proj**2 +  p_proj**2)
      call reaction_at_cm_4(M_inv, M_nucl, p_nucl, v_nucl,
     &            mass1, mass2, mass3, mass4, 
     &            p1, p2, p3, p4, v1, v2, v3, v4,
     &            N_body, N_pickup, iflag)
c if reaction does not occur in this energy it forces the original 
c particles to be scattered in s-wave
      if(iflag .eq. 1) then
         call sphrrn( v_nucl )
         Do j = 1, 3
            v_proj(j) = -v_nucl(j)
         EndDo
         call relat2(p_nucl, M_nucl, v_nucl, beta_cm, v_cm)
         call relat2(p_proj, M_proj, v_proj, beta_cm, v_cm)
         return
      endif
      call relat2(p1,mass1,v1,beta_cm,v_cm)
      call relat2(p2,mass2,v2,beta_cm,v_cm)
      call relat2(p3,mass3,v3,beta_cm,v_cm)
      call relat2(p3,mass4,v4,beta_cm,v_cm)
      return
      end



      subroutine reaction_at_cm_3(M_inv, M_nucl, p_nucl, v_nucl,
     &            mass1, mass2, mass3, p1, p2, p3, v1, v2, v3,
     &            N_body, N_pickup, iflag)
      implicit NONE
c> 3-body reaction in 3 body phase space with nuclear form factor 
c> input
c>  M_inv : total mass (^4He + kaon, ^4He + Sigma^- etc.)
c>  mass1 : mass of the doughter particle No.1
c>  mass2 : mass of the doughter particle No.2
c>  mass3 : mass of the doughter nuclei
c>  N_body : define nuclear form factor by the number of nucleon
c> output
c>  p1 : momentum of the particle No.1
c>  p2,p3 : same as p1
c>  v1 : normal vector of the particle No.1
c>  v2,v3 : same as v1
c var
      real v1(3),v2(3),v3(3),M_inv,mass1,mass2,mass3,p1,p2,p3
      real Q, c1_3, M_nucl, p_nucl, v_nucl(3), ps, vs(3)
      integer N_body, N_pickup, iflag
      integer j
      logical form_factor_check
      logical ph3_OK, form_factor_OK
c
	real tol / 0.03 /, M_np / 931. /, r_nucl3, diff_nucl3
	integer N_nucl3
      real eps / 1.0e-5/
c check
	r_nucl3 = mass3 / M_np
	N_nucl3 = r_nucl3 + tol
	diff_nucl3 = r_nucl3 - N_nucl3 
	if( diff_nucl3 .gt. tol*2. ) then
	   write(*,*) 'reaction_3 faiure'
	   write(*,*) 'particle No.3 is not a nucleon/nuclei'
	   write(*,*) 'mass1, mass2, mass3', mass1, mass2, mass3
	   stop
	endif
c begin
      Q = M_inv - mass1 - mass2 - mass3
c	write(*,*) 'M_inv', M_inv, M_nucl, p_nucl, v_nucl, Q
c	write(*,*) 'mass1, mass2, mass3', mass1, mass2, mass3
      if(Q .le. eps) then
         if(iflag .ne. -1) then
            write(*,*) ' warning +++++++ reaction_at_rest faild '
            write(*,*) ' M_inv, mass1, mass2, mass3, Q ', M_inv,mass1,mass2,mass3,Q
         endif
         iflag = 1
         p1 = 0.
         p2 = 0.
         p3 = 0.
         return
	else 
	   iflag = 0
      endif
      ph3_OK = .false.
      form_factor_OK = .false.
      Do while( .not. form_factor_OK )
         Do while( .not. ph3_OK )
            call phase_3_trial(Q, mass1, mass2, mass3, p1, p2, p3, c1_3, ph3_OK)
c	write(*,*) 'ph3_OK', ph3_OK, p1, p2, p3, c1_3
         EndDo
         if( p_nucl .eq. 0.) then
            form_factor_OK = form_factor_check(p3, N_body, N_pickup)
         else
            call sphrrn( v3 )
            ps = p3
            Do j = 1, 3
               vs(j) = v3(j)
            EndDo
            call inv_relat(ps,mass3,vs,p_nucl,M_nucl,v_nucl)
            form_factor_OK = form_factor_check(ps, N_body, N_pickup)
         endif
         ph3_OK = .false.
c	write(*,*) 'form_factor_OK', form_factor_OK, ps
	EndDo
      if( p_nucl .eq. 0.) then
         call v3set(c1_3,v1,v2,v3,p1,p2,p3)
      else
         call v3set_v3keep(c1_3,v1,v2,v3,p1,p2,p3)
      endif
      return
      end




      subroutine reaction_at_cm_4(M_inv, M_nucl, p_nucl, v_nucl,
     &            mass1, mass2, mass3, mass4,
     &            p1, p2, p3, p4, v1, v2, v3, v4,
     &            N_body, N_pickup, iflag)
      implicit NONE
c> 3-body reaction in 3 body phase space with nuclear form factor 
c> input
c>  M_inv : total mass (^4He + kaon, ^4He + Sigma^- etc.)
c>  mass1 : mass of the doughter particle No.1
c>  mass2 : mass of the doughter particle No.2
c>  mass3 : mass of the doughter particle No.3
c>  mass4 : mass of the doughter nuclei
c>  N_body : define nuclear form factor by the number of nucleon
c> output
c>  p1 : momentum of the particle No.1
c>  p2,p3 : same as p1
c>  v1 : normal vector of the particle No.1
c>  v2,v3 : same as v1
c var
      real v1(3),v2(3),v3(3),v4(3),M_inv,mass1,mass2,mass3,mass4,p1,p2,p3,p4
      real c1_3, rn, weight, M_nucl, p_nucl, v_nucl(3), pc, vc(3)
	real b_a, Q3
      integer N_body, N_pickup, iflag, j
      logical form_factor_check
      logical ph3_OK, ph4_OK, form_factor_OK
c common
      real*8 Q_tot, M_A, Q_A, M4, T4, T4_max, ro_max
      common /real8c/ Q_tot, M_A, Q_A, M4, T4, T4_max, ro_max
c
	real tol / 0.03 /, M_np / 931. /, r_nucl4, diff_nucl4
	integer N_nucl4
c check
	r_nucl4 = mass4 / M_np
	N_nucl4 = r_nucl4 + tol
	diff_nucl4 = r_nucl4 - N_nucl4 
	if( diff_nucl4 .gt. tol*2. ) then
	   write(*,*) 'reaction_4 faiure'
	   write(*,*) 'particle No.4 is not a nucleon/nuclei'
	   write(*,*) 'mass1, mass2, mass3, mass4', mass1, mass2, mass3, mass4
	   stop
	endif
c begin
      call search_4_phase_max(M_inv, mass1, mass2, mass3, mass4, iflag)
      if(iflag .eq. 1) then
         return
      endif
c
      ph4_OK = .false.
      form_factor_OK = .false.
      Do while ( .not. form_factor_OK )
         Do while ( .not. ph4_OK )
            T4 = T4_max * rn()
            Q_A = sqrt((M_A + Q_tot)**2 - 2.d0*(M_A + Q_tot + M4)*T4 ) - M_A
            weight = sqrt( T4 * (T4 + 2.*M4) ) * Q_A**2 / ro_max
            if( rn() .lt. weight ) ph4_OK = .true.
         EndDo
c
         p4 = sqrt( T4 * (T4 + 2.*M4) )
         if( p_nucl .eq. 0.) then
            form_factor_OK = form_factor_check(p4, N_body, N_pickup) 
         else
            call sphrrn( v4 )
            pc = p4
            Do j = 1, 3
               vc(j) = v4(j)
            EndDo
            call inv_relat(pc,mass4,vc,p_nucl,M_nucl,v_nucl)
            form_factor_OK = form_factor_check(pc, N_body, N_pickup) 
         endif
         ph4_OK = .false.
	EndDo
c
      b_A = - p4 / (M_A + Q_tot - T4) 
      call sphrrn( v4 )
c	if(ev_NUM .eq. ev_error) write(*,*) 'at rest reaction', p4, N_body, N_pickup
c	if(ev_NUM .eq. ev_error) write(*,*) 'cm beta for 3 body', b_A
c
      Q3 = Q_A
      ph3_OK = .false.
      Do while( .not. ph3_OK )
         call phase_3_trial(Q3, mass1, mass2, mass3, p1, p2, p3, c1_3, ph3_OK)
      EndDo
      call v3set(c1_3,v1,v2,v3,p1,p2,p3)
c
      call relat2(p1,mass1,v1,b_A,v4)
      call relat2(p2,mass2,v2,b_A,v4)
      call relat2(p3,mass3,v3,b_A,v4)
c
      return
      end



	subroutine breakup_at_rest_3(M_inv, 
     &	mass1, mass2, mass3, p1, p2, p3,  v1, v2, v3, 
     &	N_body, N_pickup, iflag)
	implicit NONE
c i/o
	real M_inv
      real v1(3),v2(3),v3(3),mass1,mass2,mass3,p1,p2,p3
	integer iflag, N_body, N_pickup
c begin
	   write(*,*) 'please use reaction_.._3 for this reaction'
	   write(*,*) 'N_body, N_pickup   ', N_body, N_pickup
	   write(*,*) 'mass1, mass2, mass3', mass1,mass2,mass3
	stop
 	end





	subroutine breakup_in_flight_3(
     &	 M_nucl, p_nucl, v_nucl, M_proj, p_proj, v_proj, 
     &	mass1, mass2, mass3, p1, p2, p3,  v1, v2, v3, 
     &	N_body, N_pickup, iflag)
	implicit NONE
c i/o
	real M_nucl, p_nucl, v_nucl(3), M_proj, p_proj, v_proj(3)
      real v1(3),v2(3),v3(3),mass1,mass2,mass3,p1,p2,p3
	integer iflag, N_body, N_pickup
c begin
	   write(*,*) 'please use reaction_.._3 for this reaction'
	   write(*,*) 'N_body, N_pickup   ', N_body, N_pickup
	   write(*,*) 'mass1, mass2, mass3', mass1,mass2,mass3
	stop
 	end





	subroutine breakup_at_rest_4(M_nucl, M_proj,  
     &	mass1, mass2, mass3, mass4, p1, p2, p3, p4, 
     &	v1, v2, v3, v4, N_body, N_pickup, iflag)
	implicit NONE
c i/o
      real v1(3),v2(3),v3(3),v4(3),mass1,mass2,mass3,mass4
      real p1,p2,p3,p4, M_inv
	integer iflag, N_body, N_pickup
c local
	real M_nucl, p_nucl, v_nucl(3), M_proj, p_proj, v_proj(3)
c
	M_inv = M_nucl + M_proj
	p_nucl = 0.
	v_nucl(1) = 0.
	v_nucl(2) = 0.
	v_nucl(3) = 1.
	p_proj = 0.
	v_proj(1) = 0.
	v_proj(2) = 0.
	v_proj(3) = 1.
	call breakup_in_flight_4(M_nucl, p_nucl, v_nucl, M_proj, p_proj, v_proj, 
     &	mass1, mass2, mass3, mass4, p1, p2, p3, p4, 
     &	v1, v2, v3, v4, N_body, N_pickup, iflag)
	return
	end





	subroutine breakup_in_flight_4(
     &	M_nucl, p_nucl, v_nucl, M_proj, p_proj, v_proj, 
     &	mass1, mass2, mass3, mass4, p1, p2, p3, p4, 
     &	v1, v2, v3, v4, N_nucl, N_pickup, iflag)
	implicit NONE
c This routine only applicable to:
c whether X + NNNN -> (X+NN) + N N -> Y + Z + N + N : N_nucl=4 & N_pickup=2,   1
c         X + NNN  -> (X+N) + N N  -> Y + Z + N + N : N_nucl=3 & N_pickup=1,   1
c It is also good for 
c         X + NNd  -> (X+N) + N d  -> Y + Z + N + d : N_nucl=4 & N_pickup=1,   2
c in the none-relativistic limit.
c The simple extentions are 
c         X + NNNA -> (X+NN) + N A -> Y + Z + N + A : N_nucl=A+3 & N_pickup=2, 3?
c   or    X + NNA  -> (X+N) + N A  -> Y + Z + N + A : N_nucl=A+2 & N_pickup=1. 2
c and those are allowded. In here, however, nuclear form factor for A is 
c not simple in X + NNNA reaction.  I just simplify by N_pickup_4 = 2!
c
c The following case;
c            X + NNA -> Y + Z + V + A : N_nucl=2+A & N_pickup=2    2
c and        X +  NA -> Y + Z + V + A : N_nucl=1+A & N_pickup=1,   1
c it calls reaction_in_flight, because the residual nuclei is equal to one. 
c i/o
	real M_nucl, p_nucl, v_nucl(3), M_proj, p_proj, v_proj(3)
      real v1(3),v2(3),v3(3),v4(3),mass1,mass2,mass3,mass4
      real p1,p2,p3,p4
	integer iflag, N_nucl, N_pickup, ierr
c local
	real M_inv, beta_cm, v_cm(3), ps, vs(3)
	real Q4, rn, b_A, Q3, c1_3
	logical ph4_OK, ph3_OK, Form_Factor_OK, form_factor_check
	real tol / 0.03 /, M_np / 931. /, weight
	real r_nucl4, r_nucl3, diff_nucl4, diff_nucl3
	integer j, N_nucl4, N_nucl3
c common
      real*8 Q_tot, M_A, Q_A, M4, T4, T4_max, ro_max
      common /real8c/ Q_tot, M_A, Q_A, M4, T4, T4_max, ro_max
c
c check reaction
	ierr = 0
	r_nucl4 = mass4 / M_np
	N_nucl4 = r_nucl4 + tol
	diff_nucl4 = r_nucl4 - N_nucl4 
	if( diff_nucl4 .gt. tol*2. ) ierr = 1
c check if its a request for reaction_in_flight
	if( N_nucl - N_pickup .eq. N_nucl4 .and. ierr .eq. 0 ) then
	   write(*,*) 'please use reaction_.._4 for this reaction'
	   write(*,*) 'N_nucl, N_pickup, N_nucl4 ', N_nucl, N_pickup, N_nucl4
	   write(*,*) 'mass1, mass2, mass3, mass4', mass1,mass2,mass3,mass4
	   stop
	endif
	r_nucl3 = mass3 / M_np
	N_nucl3 = r_nucl3 + tol
	diff_nucl3 = r_nucl3 - N_nucl3 
	if( N_nucl3 .ne. 1 .or. diff_nucl3 .gt. tol*2. ) ierr = 1
	if( N_pickup .le. 0 .or. N_pickup .ge. 3 ) ierr = 1
	if( N_nucl4 + N_pickup + 1 .ne. N_nucl ) ierr = 1
	if( ierr .eq. 1 ) then
         write(*,*) ' error +++++++ breakup 4 faild '
         write(*,*) ' unsupported reaction is requested. '
	   write(*,*) ' particle No.3 is expected to be nucleon!'
	   write(*,*) ' particle No.4 is expected to be nuceon or nuclei!'
	   write(*,*) ' N_nucl, N_pickup', N_nucl, N_pickup
	   write(*,*) ' N_nucl3 + N_nucl4 = N_nucl', N_nucl3, N_nucl4, N_nucl
	   write(*,*) ' mass3, mass4', mass3, mass4
	   stop
	endif 
	if(M_nucl.le.0.) then
	   iflag = 1
         p1 = 0.
         p2 = 0.
         p3 = 0.
         p4 = 0.
         return
	endif
c begin
      call transform2cm(p_nucl,M_nucl,v_nucl,p_proj,M_proj,v_proj,
     &		beta_cm, v_cm)
      M_inv = sqrt(M_nucl**2 + p_nucl**2) + sqrt(M_proj**2 +  p_proj**2)
      if(iflag .eq. -1) then
         call search_4_phase_max(M_inv, 
     &		mass1, mass2, mass3, mass4, iflag)
	else 
         call search_4_phase_max(M_inv, 
     &		mass1, mass2, mass3, mass4, iflag)
         if(iflag .eq. 1) then
	      Q4 = Q_tot
            write(*,*) ' warning +++++++ breakup 4 faild '
            write(*,*) ' M_inv, mass1, mass2, mass3, mass4, Q ', 
     &		M_inv,mass1,mass2,mass3,mass4,Q4
		write(*,*) ' M_nucl, p_nucl, M_proj, p_proj', 
     &		M_nucl, p_nucl, M_proj, p_proj
	   endif
	endif
      if(iflag .eq. 1) then
         p1 = 0.
         p2 = 0.
         p3 = 0.
         p4 = 0.
         call sphrrn( v_nucl )
         Do j = 1, 3
            v_proj(j) = -v_nucl(j)
         EndDo
         call relat2(p_nucl,M_nucl,v_nucl, beta_cm, v_cm)
         call relat2(p_proj,M_proj,v_proj, beta_cm, v_cm)
         return
      endif
c
      ph4_OK = .false.
      form_factor_OK = .false.
      Do while ( .not. form_factor_OK )
         Do while ( .not. ph4_OK )
            T4 = T4_max * rn()
            Q_A = sqrt((M_A + Q_tot)**2 
     &		- 2.d0*(M_A + Q_tot + M4)*T4 ) - M_A
            weight = sqrt( T4 * (T4 + 2.*M4) ) * Q_A**2 / ro_max
            if( rn() .lt. weight ) ph4_OK = .true.
         EndDo
c
         p4 = sqrt( T4 * (T4 + 2.*M4) )
         call sphrrn( v4 )
         ps = p4
         Do j = 1, 3
            vs(j) = v4(j)
         EndDo
         call inv_relat(ps,mass4,vs,p_nucl,M_nucl,v_nucl)
         form_factor_OK = form_factor_check(ps, N_nucl, N_pickup) 
	   if( form_factor_OK ) then
            b_A = - p4 / (M_A + Q_tot - T4) 
            call sphrrn( v4 )
            Q3 = Q_A
            ph3_OK = .false.
            Do while( .not. ph3_OK )
               call phase_3_trial(Q3, mass1, mass2, mass3, 
     &		p1, p2, p3, c1_3, ph3_OK)
            EndDo
            call v3set(c1_3,v1,v2,v3,p1,p2,p3)
            call relat2(p1,mass1,v1,b_A,v4)
            call relat2(p2,mass2,v2,b_A,v4)
            call relat2(p3,mass3,v3,b_A,v4)
	      ps = p3
            Do j = 1, 3
               vs(j) = v3(j)
            EndDo
            call inv_relat(ps,mass3,vs,p_nucl,M_nucl,v_nucl)
            Form_Factor_OK = form_factor_check(ps, N_nucl, N_pickup) 
	   endif
         ph4_OK = .false.
	EndDo
      call relat2(p1,mass1,v1, beta_cm, v_cm)
      call relat2(p2,mass2,v2, beta_cm, v_cm)
      call relat2(p3,mass3,v3, beta_cm, v_cm)
      call relat2(p3,mass4,v4, beta_cm, v_cm)
 	return
	end


c ..............................................................



      logical function form_factor_check(p, N_body, N_pickup)
      implicit NONE
      real p, rn, x, ref, pickup
      integer N_body, N_pickup, j
      real  Fermi_momentum(4) /0., 110., 122.5, 147.666/
c	real p_wave_enhance / 1.1 / ! Outa's data
	real p_wave_enhance / 1. /  ! Akaishi's 
	real pF
c common /F_KN/ for knucl Fermi-motion enlaging factor and SL conversion
	logical Fermi_Knucl
	real SLC_KN_Spp, SLC_KN_Spn, SLC_KN_Snn
	common /F_KN/ Fermi_Knucl, SLC_KN_Spp, SLC_KN_Spn, SLC_KN_Snn
c begin 
      if( N_body .le. 1 .or. N_body .ge. 5 
     &      .or. N_pickup.le.0 .or. N_pickup.ge.N_body) then
         write(*,*) 'Error: form_factor_check does not work'
         write(*,*) 'other than 3 or 4, at the moment N_body = ', N_body, N_pickup
         stop
      endif
      j = N_body - 1
	pickup = 1.0
	Do While (j .GE. (N_body-N_pickup+1))
	  pickup = pickup * j
	  j = j - 1
	EndDo
      j = N_pickup
	Do While (j .GE. 2)
	  pickup = pickup / j
	  j = j - 1
	EndDo
c	write(*,*) pickup, ' pickup'
	if(Fermi_Knucl) pickup = pickup * 2.5
	pF = Fermi_momentum(N_body) * p_wave_enhance
      x = ( p / pF )**2
      ref = ( exp( -x/pickup ) + 0.00286 * exp(  -x/12./pickup ) ) / 1.00286
      if( rn() .gt. ref ) then
         form_factor_check = .false.
      else
         form_factor_check = .true.
      endif
      return
      end





      subroutine transform2cm(p1,m1,v1,p2,m2,v2,beta_cm,v_cm)
c< transform particle 1 and particle 2 to the center of mass system
	implicit none
c var
	real v1(3),v2(3),m1,m2,p1,p2, beta_cm, v_cm(3)
	real e1, e2, s
	integer j
c check
	if( p1.eq.0. .and. p2.eq.0. ) then
	   beta_cm = 0.
	   return
	endif
c begin
      s = 0.
      Do j = 1, 3
         v_cm(j) = p1*v1(j) + p2*v2(j)
         s = s + v_cm(j)**2
      enddo
      s = sqrt(s)
c      if(ev_NUM .ge. ev_error) write(*,*) 's = ', s
      if(s .lt. 1.e-5) then
         beta_cm = 0.
         v_cm(1) = 1.
         v_cm(2) = 0.
         v_cm(3) = 0.
      else
         E1 = sqrt(p1**2+m1**2)
         E2 = sqrt(p2**2+m2**2)
c      if(ev_NUM .ge. ev_error) write(*,*) 'E1, E2 = ', E1, E2
         beta_cm = s / ( E1 + E2 )
         do j = 1, 3
            v_cm(j) = v_cm(j) / s
         enddo
c      if(ev_NUM .ge. ev_error) write(*,*) 'v_cm = ', v_cm
         call inv_relat2(p1,m1,v1,beta_cm,v_cm)
         call inv_relat2(p2,m2,v2,beta_cm,v_cm)
c      if(ev_NUM .ge. ev_error) write(*,*) 'inv_relat2 end'
      endif
      return
      end



      subroutine inv_transform2cm(p1,m1,v1,p2,m2,v2,beta_cm,v_cm)
c< transform particle 1 and particle 2 to the center of mass system
	implicit none
c var
	real v1(3),v2(3),m1,m2,p1,p2, beta_cm, v_cm(3)
	if( beta_cm .eq. 0. ) return
c begin
	call relat2(p1,m1,v1,beta_cm,v_cm)
	call relat2(p2,m2,v2,beta_cm,v_cm)
      return
      end




      subroutine relat(p1,m1,v1,p0,m0,v0)
c< transform the particle 1 to the particle 0 system (boost to v0 direction)
c boost to v0 direction
      implicit none
c var
      real v1(3),v0(3),m1,m0,p1,p0
      real e0, e1, r, p1z, s
      integer j
	if( p0 .eq. 0. ) return
	if( p1 .eq. 0. .and. m1 .eq. 0.) return ! un-physical dummy particle
c begin
      E0 = sqrt(p0**2+m0**2)
      E1 = sqrt(p1**2+m1**2)
      r = 0.
      Do j = 1, 3
        r = r + v0(j) * v1(j)
      EndDo
      p1z = (E0*p1*r + p0*E1)/m0
      E1  = (p0*p1*r + E0*E1)/m0
      s = 0.
      Do j = 1, 3
        v1(j) = p1*(v1(j)-r*v0(j))+p1z*v0(j)
        s = s + v1(j)**2
      EndDo
      p1 = sqrt(s)
      Do j = 1, 3
        v1(j) = v1(j)/p1
      EndDo
      return
      end


      subroutine inv_relat(p1,m1,v1,p0,m0,v0)
c< particle 1 ovserved from particle 0
c boost to v0 direction
       implicit none
c var
       real v1(3),v0(3),m1,m0,p1,p0
       real e0, e1, r, p1z, s
       integer j
	if( p0 .eq. 0. ) return
	if( p1 .eq. 0. .and. m1 .eq. 0.) return ! un-physical dummy particle
c begin
      E0 = sqrt(p0**2+m0**2)
      E1 = sqrt(p1**2+m1**2)
      r = 0.
      Do j = 1, 3
        r = r + v0(j) * v1(j)
      EndDo
      p1z = (  E0*p1*r - p0*E1 )/m0
      E1  = ( -p0*p1*r + E0*E1 )/m0
      s = 0.
      Do j = 1, 3
        v1(j) = p1*(v1(j)-r*v0(j))+p1z*v0(j)
        s = s + v1(j)**2
      EndDo
      p1 = sqrt(s)
      Do j = 1, 3
        v1(j) = v1(j)/p1
      EndDo
      return
      end


      subroutine relat2(p1,m1,v1,beta,v_beta)
c< transform the particle 1 to the beta (boost to v_beta direction)
       implicit none
c var
       real v1(3),v_beta(3),m1,p1, beta
       real e1, r, p1z, s
       integer j
	if( beta .eq. 0. ) return
	if( p1 .eq. 0. .and. m1 .eq. 0.) return ! un-physical dummy particle
c begin
      E1 = sqrt(p1**2+m1**2)
      r = 0.
      Do j = 1, 3
        r = r + v_beta(j) * v1(j)
      EndDo
      p1z = ( p1*r + beta*E1 ) / sqrt( 1. - beta**2 )
      E1  = ( beta*p1*r + E1 ) / sqrt( 1. - beta**2 )
      s = 0.
      Do j = 1, 3
        v1(j) = p1*(v1(j) - r*v_beta(j)) + p1z*v_beta(j)
        s = s + v1(j)**2
      EndDo
      p1 = sqrt(s)
      Do j = 1, 3
        v1(j) = v1(j)/p1
      EndDo
      return
      end


      subroutine inv_relat2(p1,m1,v1,beta,v_beta)
c< particle 1 representation in beta, v_beta system
       implicit none
c var
       real v1(3),v_beta(3),m1,p1, beta
       real e1, r, p1z, s
       integer j
	if( beta .eq. 0. ) return
	if( p1 .eq. 0. .and. m1 .eq. 0.) return ! un-physical dummy particle
c begin
      E1 = sqrt(p1**2+m1**2)
      r = 0.
      Do j = 1, 3
        r = r + v_beta(j) * v1(j)
      EndDo
      p1z = (   p1*r - beta*E1 )/sqrt(1.-beta**2)
      E1  = ( - beta*p1*r + E1 )/sqrt(1.-beta**2)
      s = 0.
      Do j = 1, 3
        v1(j) = p1*(v1(j) - r*v_beta(j)) + p1z*v_beta(j)
        s = s + v1(j)**2
      EndDo
      p1 = sqrt(s)
      Do j = 1, 3
        v1(j) = v1(j)/p1
      EndDo
      return
      end



